@@ -1,10 +1,10 @@
 use std::fmt;
 
-use crate::parse_contract_addr;
 use cosmwasm_std::{
     to_binary, Addr, Attribute, BankMsg, Binary, Coin, CosmosMsg, Event, SubMsgExecutionResponse,
     WasmMsg,
 };
+use cw0::{parse_execute_response_data, parse_instantiate_response_data};
 use schemars::JsonSchema;
 use serde::Serialize;
 
@@ -91,11 +91,13 @@ where
             label: label.into(),
         };
         let res = self.execute(sender, msg.into())?;
-        parse_contract_addr(&res.data)
+        let data = parse_instantiate_response_data(res.data.unwrap_or_default().as_slice())?;
+        Ok(Addr::unchecked(data.contract_address))
     }
 
     /// Execute a contract and process all returned messages.
-    /// This is just a helper around execute()
+    /// This is just a helper around execute(),
+    /// but we parse out the data field to that what is returned by the contract (not the protobuf wrapper)
     fn execute_contract<T: Serialize>(
         &mut self,
         sender: Addr,
@@ -109,7 +111,11 @@ where
             msg,
             funds: send_funds.to_vec(),
         };
-        self.execute(sender, msg.into())
+        let mut res = self.execute(sender, msg.into())?;
+        res.data = res
+            .data
+            .and_then(|d| parse_execute_response_data(d.as_slice()).unwrap().data);
+        Ok(res)
     }
 
     /// Migrate a contract. Sender must be registered admin.
